@@ -22,7 +22,6 @@
 from sys import maxsize
 import _cdmpyparser
 
-
 def trim_docstring(docstring):
     """Taken from http://www.python.org/dev/peps/pep-0257/"""
     if not docstring:
@@ -57,7 +56,7 @@ def trim_docstring(docstring):
     return '\n'.join(lines)
 
 
-class ModuleInfoBase(object):
+class ModuleInfoBase:
 
     """Common part for the module information"""
 
@@ -224,22 +223,28 @@ class Decorator(ModuleInfoBase):
         """Provides a name for display purpose"""
         displayName = self.name
         if self.arguments:
-            displayName += "( " + ", ".join(self.arguments) + " )"
+            displayName += "(" + ", ".join(self.arguments) + ")"
         return displayName
 
 
-class Docstring(object):
+class Docstring:
 
     """Holds a docstring information"""
 
-    __slots__ = ["line", "text"]
+    __slots__ = ["startLine", "endLine", "line", "text"]
 
-    def __init__(self, text, line):
-        self.line = line
+    def __init__(self, text, startLine, endLine):
+        self.startLine = startLine
+        self.endLine = endLine
         self.text = text
 
+        # Compatibility: modules for 3.7 and below reported the end line only
+        # under the 'line' name
+        self.line = endLine
+
     def __str__(self):
-        return "Docstring[" + str(self.line) + "]: '" + self.text + "'"
+        return "Docstring[" + str(self.startLine) + ":" + \
+            str(self.endLine) + "]: '" + self.text + "'"
 
 
 class Argument:
@@ -331,12 +336,10 @@ class Function(ModuleInfoBase):
         first = True
         for arg in self.arguments:
             if first:
-                displayName += " " + str(arg)
+                displayName += str(arg)
                 first = False
             else:
                 displayName += ", " + str(arg)
-        if self.arguments:
-            displayName += " "
         displayName += ")"
         if self.returnAnnotation is not None:
             displayName += ' -> ' + self.returnAnnotation
@@ -400,11 +403,11 @@ class Class(ModuleInfoBase):
         " Provides a name for display purpose "
         displayName = self.name
         if self.base:
-            displayName += "( " + ", ".join(self.base) + " )"
+            displayName += "(" + ", ".join(self.base) + ")"
         return displayName
 
 
-class BriefModuleInfo(object):
+class BriefModuleInfo:
 
     """Holds a single module content information"""
 
@@ -574,13 +577,14 @@ class BriefModuleInfo(object):
         """Memorizes a decorator argument"""
         self.__lastDecorators[-1].arguments.append(name)
 
-    def _onDocstring(self, docstr, line):
+    def _onDocstring(self, docstr, startLine, endLine):
         """Memorizes a function/class/module docstring"""
         if self.objectsStack:
             self.objectsStack[-1].docstring = \
-                Docstring(trim_docstring(docstr), line)
+                Docstring(trim_docstring(docstr), startLine, endLine)
         else:
-            self.docstring = Docstring(trim_docstring(docstr), line)
+            self.docstring = Docstring(trim_docstring(docstr),
+                                       startLine, endLine)
 
     def _onArgument(self, name, annotation):
         """Memorizes a function argument"""
